﻿/**

///	レイヤー群にひとつのアクションを順次適用していくスクリプト

	制作：彩藤神楽	(http://www.re-sonance.com/)


	Copyright (C) 2007-2008 Saitoh Kagura, All Rights Reserved.

 */

const SCRIPT_TITLE	= "アクション一括適用";
const VERSION		= "0.1.00";


///-------------------------------------------------------------
///	外部スクリプトファイル読み込み
//--------------------------------------------------------------
#include "./lib/Resonance.jsxinc"
#include "./lib/ScriptUtil.jsxinc"
#include "./lib/StringUtil.jsxinc"
#include "./lib/LayerUtil.jsxinc"
#include "./lib/ConfigDialog.jsxinc"


///-------------------------------------------------------------
///	グローバル
//--------------------------------------------------------------

//--> 基礎準備
#target photoshop
app.bringToFront();
const NEW_LINE	= Resonance.NEW_LINE;
const NEW_LINE2	= NEW_LINE + NEW_LINE;
const INDENT	= Resonance.INDENT;
const INDENT2	= INDENT + INDENT;
_resonance		= new Resonance(SCRIPT_TITLE, VERSION);

///> 確認ダイアログ
const MESSAGE_CONFIRM_PREFIX	= "アクションを";
const MESSAGE_CONFIRM_BODY		= "に適用します。" + NEW_LINE + "適用先は【 ";
const MESSAGE_CONFIRM_SUFFIX	= " 】内です。" + NEW_LINE2 + "よろしいですか？";
const TITLE_RESULT		= "レイヤーソート完了";
const MESSAGE_RESULT	= "並び替えが完了しました。";
const MESSAGE_SKIP		= "並び替えの必要はありませんでした。";

///> エラーメッセージ定数
const ERROR_ACTION_EMPTY		= "使用するアクション名を入力して下さい。";

//> グローバルに扱う設定系オブジェクト
var _config;

///> 設定データ
const INDEX_LAYER_ONLY		= 0;
const INDEX_LAYER_SET_ONLY	= 1;
const INDEX_ALL_LAYER		= 2;
var _configData	= {};
_configData.infoData	= {
	 _targetPrefix	: "対象範囲【 "
	,_targetSuffix	: " 】"
	,Target			: ["レイヤーのみ", "レイヤーセットのみ", "レイヤー及びレイヤーセット"]
	,_type			: "対象の種類"
	,_other			: "詳細設定"
	,_Filter		: "レイヤー名フィルタ:"
	,AsRecursive	: "サブレイヤーセットにも適用する"
	,AsNonstop		: "アクション実行時のエラーを無視する"
	,Action	: {
		 _FileName		: "アクションファイル名:"
		,_ForLayer		: "レイヤーに対するアクション:"
		,_ForLayerSet	: "レイヤーセットに対するアクション:"
	}
};
_configData.helpData	= {
	 Filter		: "JavaScriptの正規表現でマッチするレイヤー名のレイヤーのみを対象とします。"
};
_configData.defaultData	= {
	 ScriptTitle	: SCRIPT_TITLE
	,ScriptVersion	: VERSION
	,Target			: 0
	,Filter			: ""
	,AsRecursive	: false
	,AsNonstop		: false
	,Action	: {
		 FileName	: ""
		,ForLayer	: ""
		,ForLayerSet: ""
		,FileNameHistory	: []
		,ForLayerHistory	: []
		,ForLayerSetHistory	: []
	}
};
_configData.widthData	= {
	 _target		: 300
	,Filter			: 160
	,Action	: {
		 "FileName,ForLayer,ForLayerSet"	: 160
		,"FileNameHistory,ForLayerHistory,ForLayerSetHistory"	: 100
	}
};
_configData.heightData	= {
	 "_type,_other"	: 130
}


//--> 本体実行
main();


///-------------------------------------------------------------
///	メインフロー関数
//--------------------------------------------------------------

///{} 本体
function main() {

	//--? 起動時チェック
	try{
		ScriptUtil.inspectDocumentExists();
		ScriptUtil.inspectPhotoshopVersion();
		ScriptUtil.inspectDocumentFormat();
	}catch(e){
		_resonance.showError(e);
		return;
	}

	try{

		mainFlow();

	}catch(e){
		_resonance.showError(e);
	}

};


///{} メインフロー
function mainFlow() {

	//--> 設定読み込み
	loadConfig();

	//--> 設定ダイアログ表示
	if(_config.show()){

		//(!) OKの場合

		doActionForLayers();

	}

};


///{} 設定読み込みその編集用ダイアログを配置
function loadConfig() {

	//--> XMLデータとダイアログ設定を関連付けるクラスを生成
	_config	= new ConfigDialog(_configData);
	ScriptUtil.tries(_resonance.windowTitle, _config.connectXMLON, _config, [
		 _resonance.configFilePath
		,SCRIPT_TITLE
		,VERSION
	]);

	_config.title	= _resonance.windowTitle;

  _config.prepare	= function(){

	this.global.targetParent	= LayerUtil.getTargetParentLayer(activeDocument.activeLayer);

	/// レイアウト

	const column_opt	= this.setOptions({orientation:"column"});
	const row_opt		= this.setOptions({orientation:"row"});
	const main_opt		= this.setOptions({orientation:"column", spacing:ConfigDialog.BROAD_SPACE});
	const edit_opt		= this.setOptions({orientation:"row", alignment:"right", spacing:ConfigDialog.NARROW_SPACE});
	const left_opt		= this.setOptions({orientation:"column", alignChildren:"left"});

	//--> メインウィンドウ生成
	this.create();

	//--> メイングループ
	this.addGroup("_main", column_opt);
		this.addPanel("_main/_target", main_opt, this.infoData._targetPrefix + StringUtil.getFullName(this.global.targetParent, "/", "LayerSet") + this.infoData._targetSuffix);

			//--> アクション設定
			this.addGroup("_main/_target/Action", column_opt);
				this.addGroup("_main/_target/Action/_set", edit_opt);
					this.addText("_FileName");
					var file_etx	= this.addEditText("FileName", undefined, "onChangeActionFile");
					var fileHistory_ddl	= this.addDropDownListMenu("FileNameHistory", undefined, "onChangeFileNameHistory");
				this.addGroup("_main/_target/Action/_layer", edit_opt);
					this.addText("_ForLayer");
					var forLayer_etx	= this.addEditText("ForLayer", undefined, "onChangeActionForLayer");
					var layerHistory_ddl	= this.addDropDownListMenu("ForLayerHistory", undefined, "onChangeLayerHistory");
				this.addGroup("_main/_target/Action/_layerSet", edit_opt);
					this.addText("_ForLayerSet");
					var forLayerSet_etx	= this.addEditText("ForLayerSet", undefined, "onChangeActionForLayerSet");
					var layerSetHistory_ddl	= this.addDropDownListMenu("ForLayerSetHistory", undefined, "onChangeLayerSetHistory");

			this.addGroup("_main/_target/_custom", row_opt);

				//--> 対象の種類
				this.addPanel("_main/_target/_custom/_type");
					this.addGroup("_main/_target/_custom/_type/Target", left_opt);
						this.addRadioButtonItem();
						this.addRadioButtonItem();
						this.addRadioButtonItem();

				//--> その他設定
				this.addPanel("_main/_target/_custom/_other", left_opt);
					this.addCheckBox("AsRecursive");
					this.addCheckBox("AsNonstop");
					this.addText("_Filter");
					this.addEditText("Filter");

	this.changeParentGroup("_main/");
	this.addConfirmCheckBox();
	this.addSpace();

	//--> 基本ボタン
	this.addMainButton("_main/_button", row_opt, true, true);

	/// イベントハンドラ

	var eventListener	= {};

	//{a} 履歴用配列に追加
	var setHistory = function(a, s){
		if(!s){
			return;
		}
		var max	= a.length - 1;
		if(max >= 0){
			for(var i = max; i >= 0; i--){
				if(a[i] == s){
					a.splice(i, 1);
				}
			}
		}
		a.unshift(s);
		if(a.length > ConfigDialog.MAX_HISTORY_LENGTH){
			a.pop();
		}
	};

	//e{} 実行アクションファイル変更イベント
	eventListener.onChangeActionFile	= function(conf, item){
		item.text	= item.text.replace(/\.atn$/i, "");
		setHistory(conf.data.Action.FileNameHistory, item.text);
		fileHistory_ddl.setByArray(conf.data.Action.FileNameHistory);
	};

	//e{} レイヤー用実行アクション変更イベント
	eventListener.onChangeActionForLayer	= function(conf, item){
		setHistory(conf.data.Action.ForLayerHistory, item.text);
		layerHistory_ddl.setByArray(conf.data.Action.ForLayerHistory);
	};

	//e{} レイヤーセット用実行アクション変更イベント
	eventListener.onChangeActionForLayerSet	= function(conf, item){
		setHistory(conf.data.Action.ForLayerSetHistory, item.text);
		layerSetHistory_ddl.setByArray(conf.data.Action.ForLayerSetHistory);
	};

	//e{} 実行アクションファイル履歴変更イベント
	eventListener.onChangeFileNameHistory	= function(conf, item){
		if(item.selection.text){
			conf.data.Action.FileName	= file_etx.text	= item.selection.text;
		}
	};

	//e{} 実行アクションファイル履歴変更イベント
	eventListener.onChangeLayerHistory	= function(conf, item){
		if(item.selection.text){
			conf.data.Action.ForLayer	= forLayer_etx.text	= item.selection.text;
		}
	};

	//e{} 実行アクションファイル履歴変更イベント
	eventListener.onChangeLayerSetHistory	= function(conf, item){
		if(item.selection.text){
			conf.data.Action.ForLayerSet	= forLayerSet_etx.text	= item.selection.text;
		}
	};

	//e{} 対象レイヤー取得ボタンイベント
	eventListener.onClickOK	= function(conf, item){

		var Action	= conf.data.Action;
		var target	= conf.data.Target;

		if(
			!Action.FileName
			||
			(
				(!Action.ForLayer && !Action.ForLayerSet)
				||
				(!Action.ForLayer && target == INDEX_LAYER_ONLY)
				||
				(!Action.ForLayerSet && target == INDEX_LAYER_SET_ONLY)
				||
				((!Action.ForLayer || !Action.ForLayerSet) && target == INDEX_ALL_LAYER)
			)
		){
			throw new Error(ERROR_ACTION_EMPTY);
		}

		var ActionInfo	= conf.infoData.Action;
		conf.confirmMessage	= MESSAGE_CONFIRM_PREFIX + conf.getListValue("Target") + MESSAGE_CONFIRM_BODY +
			conf.global.targetParent.name +
			MESSAGE_CONFIRM_SUFFIX;

		return true;

	};

	//--> イベントリスナーを登録
	this.setListener(eventListener);

  };

};


///{} レイヤー群にアクション一括適用
function doActionForLayers() {

	var data		= _config.data;

	var Action		= data.Action;
	var actionSet	= Action.FileName;
	var layerAct	= Action.ForLayer;
	var layerSetAct	= Action.ForLayerSet;
	var Target		= data.Target;

	var AsRecursive	= data.AsRecursive;
	var AsNonstop	= data.AsNonstop;

	var filter_reg	= data.Filter ? new RegExp(data.Filter) : /.*/;

	var f	= function(parent){

		var baseLayers	= parent.layers;

		var layers	= [];
		var len		= baseLayers.length;

		//{--} アクションによってレイヤー構造が変わってもいいように処理用配列に対象レイヤーを格納
		for(var i = 0; i < len; i++){
			layers[i]	= baseLayers[i];
		}

		//{--} 各レイヤーにアクション適用
		var layer;
		var isLayerSet;
		for(i = 0; i < len; i++){
			layer	= layers[i];
			activeDocument.activeLayer	= layer;
			isLayerSet	= (layer.typename == "LayerSet");
			if(
				filter_reg.test(layer.name)
				&&
				(
					(isLayerSet && (Target != INDEX_LAYER_ONLY))
					||
					(!isLayerSet && (Target != INDEX_LAYER_SET_ONLY))
				)
			){
				try{
					doAction(
						 (isLayerSet ? layerSetAct : layerAct)
						,actionSet
					);
				}catch(e){
					if(!AsNonstop){
						throw e;
					}
				}
			}
			if(isLayerSet && AsRecursive){
				f(layer);
			}
		}

	};

	f(_config.global.targetParent);

};


