﻿/**

///	レイヤー名毎に特定のレイヤーセットへ振り分けるスクリプト

	制作：彩藤神楽	(http://www.re-sonance.com/)


	Copyright (C) 2007-2008 Saitoh Kagura, All Rights Reserved.

 */

const SCRIPT_TITLE	= "レイヤー分類";
const VERSION		= "0.1.00";


///-------------------------------------------------------------
///	外部スクリプトファイル読み込み
//--------------------------------------------------------------
#include "./lib/Resonance.jsxinc"
#include "./lib/ScriptUtil.jsxinc"
#include "./lib/FileUtil.jsxinc"
#include "./lib/LayerUtil.jsxinc"
#include "./lib/StringUtil.jsxinc"
#include "./lib/ConfigDialog.jsxinc"


///-------------------------------------------------------------
///	グローバル
//--------------------------------------------------------------

//--> 基礎準備
#target photoshop
app.bringToFront();
_resonance		= new Resonance(SCRIPT_TITLE, VERSION);

//> リネームで空文字になる場合に元レイヤー名に付ける接頭辞
const THROUGH_PREFIX	= "//";

//> グローバルに扱う設定系オブジェクト
var _config;

///> 設定データ
var _configData	= {};
_configData.infoData	= {
	 _target		: "分類対象レイヤーの親階層"
	,_Delimiter		: "分類用 区切り記号:"
	,Renamed		: "レイヤー名を階層に応じて変更する"
};
_configData.defaultData	= {
	 ScriptTitle	: SCRIPT_TITLE
	,ScriptVersion	: VERSION
	,Delimiter		: "-_ ,.:;|/　・"
	,Renamed		: false
};
_configData.widthData	= {
	 _target	: 260
	,Delimiter	: 200
};
_configData.heightData	= {
};


//--> 本体実行
main();


///-------------------------------------------------------------
///	メインフロー関数
//--------------------------------------------------------------

///{} 本体
function main() {

	//--? 起動時チェック
	try{
		ScriptUtil.inspectDocumentExists();
		ScriptUtil.inspectPhotoshopVersion();
		ScriptUtil.inspectDocumentFormat();
	}catch(e){
		_resonance.showError(e);
		return;
	}

	try{

		mainFlow();

	}catch(e){
		_resonance.showError(e);
	}

};


///{} メインフロー
function mainFlow() {

	//--> 設定読み込み
	loadConfig();

	//--> 設定ダイアログ表示
	if(_config.show()){

		//(!) OKの場合

		//--> レイヤーセットに振り分け
		apportionToLayerSets();

	}

};


///{} 設定読み込みその編集用ダイアログを配置
function loadConfig() {

	//--> XMLデータとダイアログ設定を関連付けるクラスを生成
	_config	= new ConfigDialog(_configData);
	ScriptUtil.tries(_resonance.windowTitle, _config.connectXMLON, _config, [
		 _resonance.configFilePath
		,SCRIPT_TITLE
		,VERSION
	]);

	_config.title	= _resonance.windowTitle;

  ///{} ダイアログのレイアウトと設定データの対応を準備する
  _config.prepare	= function(){

	//> 分類対象の親アイテム
	this.global.targetParent	= LayerUtil.getTargetParentLayer(activeDocument.activeLayer);

	/// レイアウト

	const column_opt	= this.setOptions({orientation:"column", alignment:"top"});
	const row_opt		= this.setOptions({orientation:"row"});

	//--> メインウィンドウ生成
	this.create();

	//--> メイングループ
	this.addGroup("_main", column_opt);

	//--> 振り分け条件編集
	var parentFullName	= _resonance.documentName + "/" +
		(this.global.targetParent.typename == "Document") ?
			""
			:
			StringUtil.getFullName(this.global.targetParent, "/", "LayerSet");
	this.addPanel("_main/_target", undefined, parentFullName);
		this.addGroup("_main/_target/_delimiter", row_opt);
			this.addText("_Delimiter");
			this.addEditText("Delimiter");
		this.addGroup("_main/_target/_button", row_opt);
			this.addCheckBox("Renamed");

	//--> メインボタン
	this.addMainButton("_main/_button", row_opt, true);
		this.addResetButton();
//		this.addScriptInfoButton();

  };

};


///{} レイヤーセットに振り分け
function apportionToLayerSets() {

	var parent		= _config.global.targetParent;

	var Renamed		= _config.data.Renamed;

	var Delimiter	= _config.data.Delimiter;
	if((/\-/).test(Delimiter)){
		Delimiter	= "-" + Delimiter.replace(/\-/g, "");
	}
	Delimiter	= Delimiter.replace(/\[/, "\\[").replace(/\]/, "\\]");

	var name_reg		= new RegExp("[^" + Delimiter + "]+");
	var delimiter_reg	= new RegExp("[" + Delimiter + "]+");
	var artLayers	= parent.artLayers;
	var layerSets	= parent.layerSets;
	var layer;
	var visible;
	var layerSet;
	var match;
	var name;
	var rename;
	var max	= artLayers.length - 1;
	if(max >= 0){
		//{--} 下から順に処理
		for(var i = max; i >= 0; i--){
			layer	= artLayers[i];
			match	= layer.name.match(name_reg);
			if(match){
				//(!) 振り分け対象レイヤー
				visible	= layer.visible;
				name	= match[0];
				//--> 振り分け先レイヤーセット取得・なければ作成
				try{
					layerSet	= layerSets.getByName(name);
				}catch(e){
					layerSet		= layerSets.add();
					layerSet.name	= name;
				}
				if(Renamed){
					rename	= layer.name.replace(name_reg, "").replace(delimiter_reg, "");
					layer.name	= rename ? rename : THROUGH_PREFIX;
				}
				layer.move(layerSet, ElementPlacement.INSIDE);
				layer.visible	= visible;
			}
		}
	}

};



