﻿/**

///	特定の名前のレイヤーを検索するスクリプト

	制作：彩藤神楽	(http://www.re-sonance.com/)


	Copyright (C) 2007-2008 Saitoh Kagura, All Rights Reserved.

 */

const SCRIPT_TITLE	= "レイヤー検索";
const VERSION		= "0.1.00";


///-------------------------------------------------------------
///	外部スクリプトファイル読み込み
//--------------------------------------------------------------
#include "./lib/Resonance.jsxinc"
#include "./lib/ScriptUtil.jsxinc"
#include "./lib/StringUtil.jsxinc"
#include "./lib/LayerUtil.jsxinc"
#include "./lib/ConfigDialog.jsxinc"


///-------------------------------------------------------------
///	グローバル
//--------------------------------------------------------------

//--> 基礎準備
#target photoshop
app.bringToFront();
const NEW_LINE	= Resonance.NEW_LINE;
const NEW_LINE2	= NEW_LINE + NEW_LINE;
_resonance		= new Resonance(SCRIPT_TITLE, VERSION);

///> 確認ダイアログ
const TITLE_RESULT			= "レイヤー検索完了";
const MESSAGE_RESULT_PREFIX	= "以下のレイヤーが見つかりました。" + NEW_LINE2;
const MESSAGE_RESULT_SET_PREFIX	= "以下のレイヤーセットが見つかりました。" + NEW_LINE2;

///> エラーメッセージ
const ERROR_NOT_FOUND_SUFFIX	= "は見つかりませんでした。";
const ERROR_NAME_EMPTY			= "検索するレイヤー名を入力して下さい。";
const ERROR_REG_WORD			= "正規表現の構文が不正です。" + NEW_LINE2;

//> グローバルに扱う設定系オブジェクト
var _config;

///> 設定データ
var _configData	= {};
_configData.infoData	= {
	 _searchPrefix	: "検索範囲【 "
	,_searchSuffix	: " 】"
	,AsRegular		: "正規表現を使用(&R)"
	,AsAllLayers	: "全体を検索対象にする(&T)"
	,RigidlyInspected	: "レイヤー名完全一致(&I)"
	,AsSubLayerSets	: "サブレイヤーセット内も検索(&L)"
	,VisibleOnly	: "表示レイヤーのみ対象にする(&V)"
	,Target			: ["レイヤー及びレイヤーセット", "レイヤーのみ", "レイヤーセットのみ"]
	,_Target		: "検索対象:"
	,_SearchName	: "レイヤー名:"
	,_searchButton	: "検索"
};
_configData.helpData	= {
	 AsRegular		: "JavaScriptの正規表現でレイヤー名を検索します。"
};
_configData.defaultData	= {
	 ScriptTitle	: SCRIPT_TITLE
	,ScriptVersion	: VERSION
	,SearchName		: ""
	,AsRegular		: false
	,AsAllLayers	: false
	,AsSubLayerSets	: false
	,RigidlyInspected	: false
	,VisibleOnly	: false
	,History		: []
	,Target			: 0
};
_configData.listData	= {
	 Target			: ["", "ArtLayer", "LayerSet"]
};
_configData.widthData	= {
	 SearchName	: 160
	,"_search,_progressInfo,_progress"	: 340
	,History	: 100
	,_hr	: 310
};


//--> 本体実行
main();


///-------------------------------------------------------------
///	メインフロー関数
//--------------------------------------------------------------

///{} 本体
function main() {

	//--? 起動時チェック
	try{
		ScriptUtil.inspectDocumentExists();
		ScriptUtil.inspectPhotoshopVersion();
		ScriptUtil.inspectDocumentFormat();
	}catch(e){
		_resonance.showError(e);
		return;
	}

	try{

		mainFlow();

	}catch(e){
		_resonance.showError(e);
	}

};


///{} メインフロー
function mainFlow() {

	//--> 設定読み込み
	loadConfig();

	//--> 設定ダイアログ表示
	_config.show()

};


///{} 設定読み込みその編集用ダイアログを配置
function loadConfig() {

	//--> XMLデータとダイアログ設定を関連付けるクラスを生成
	_config	= new ConfigDialog(_configData);
	ScriptUtil.tries(_resonance.windowTitle, _config.connectXMLON, _config, [
		 _resonance.configFilePath
		,SCRIPT_TITLE
		,VERSION
	]);

	_config.title	= _resonance.windowTitle;

  _config.prepare	= function(){

	this.global.targetParent	= LayerUtil.getTargetParentLayer(activeDocument.activeLayer);

	/// レイアウト

	const column_opt	= this.setOptions({orientation:"column"});
	const row_opt		= this.setOptions({orientation:"row"});
	const narrowRow_opt	= this.setOptions({orientation:"row", spacing:ConfigDialog.NARROW_SPACE});

	//--> メインウィンドウ生成
	this.create();

	//--> メイングループ
	var allTarget		= this.infoData._searchPrefix + activeDocument.name + this.infoData._searchSuffix;
	var parentTarget	= this.infoData._searchPrefix + StringUtil.getFullName(this.global.targetParent, "/", "LayerSet") + this.infoData._searchSuffix;
	this.addGroup("_main", column_opt);
		this.addCheckBox("AsAllLayers", {alignment:"left"}, "onClickAllLayers");
		var search_pnl	= this.addPanel("_main/_search", column_opt, this.data.AsAllLayers ? allTarget : parentTarget);
			this.addGroup("_main/_search/_edit", narrowRow_opt);
				this.addText("_SearchName");
				var search_etx	= this.addEditText("SearchName");
				var history_ddl	= this.addDropDownListMenu("History", undefined, "onChangeHistory");
			this.ascend();
		this.addHorizon("_hr");
			this.addGroup("_main/_search/_target", row_opt);
				this.addText("_Target");
				this.addDropDownList("Target");
			this.addGroup("_main/_search/_custom", {orientation:"column", alignChildren:"left"});
				this.addCheckBox("RigidlyInspected");
				this.addCheckBox("AsRegular");
				this.addCheckBox("AsSubLayerSets");
				this.addCheckBox("VisibleOnly");

	//--> 進捗グループ
	this.addGroup("_main/_progress", column_opt);
		var search_stx	= this.addText("_progressInfo");
		var search_prg	= this.addProgressBar("_progress");
		search_prg.info_item	= search_stx;

	//--> 基本ボタン
	this.addGroup("_main/_button", row_opt);
		this.addButton("_searchButton", undefined, "onClickSearch", undefined, undefined, "defaultElement");
		this.addCancelButton();
		this.addApplyButton();
		this.addResetButton();


	/// イベントハンドラ

	var eventListener	= {};

	//e{} 検索対象全体切り替えボタンイベント
	eventListener.onClickAllLayers	= function(conf, item){
		search_pnl.text	= item.value ? allTarget : parentTarget;
	};

	//e{} 検索履歴変更イベント
	eventListener.onChangeHistory	= function(conf, item){
		if(item.selection.text){
			conf.data.SearchName	= search_etx.text	= item.selection.text;
		}
	};

	///e{} 検索ボタンイベント
	eventListener.onClickSearch	= function(conf, item){
		try{
			history_ddl.addHistory(search_etx.text);
			conf.save();
			search_prg.onStartProgress();
			searchLayer(search_prg);
			search_prg.onCompleteProgress();
			conf.close(true);
		}catch(e){
			if(!ConfigDialog.isCancelProgressError(e)){
				conf.alertAs(e, Resonance.title.ERROR, true);
			}
			search_prg.onProgressError();
		}
	};

	//--> イベントリスナーを登録
	this.setListener(eventListener);

	//--? 直実行専用処理
	if(this.data.directly){
		this.dispatchEvent("onClickSearch");
	}

  };

};


///{} レイヤー検索
function searchLayer(eventHandler) {

	var data	= _config.data;

	if(!data.SearchName){
		throw new Error(ERROR_NAME_EMPTY);
	}

	//--? 検索条件設定
	var word	= data.SearchName;
	var option	= "i";
	if(!data.AsRegular){
		word	= StringUtil.escapeRegExp(word);
	}
	if(data.RigidlyInspected){
		word	= "^" + word + "$";
		option	= "";
	}
	try{
		var search_reg	= new RegExp(word, option);
	}catch(e){
		throw new Error(ERROR_REG_WORD + '"' + data.SearchName + '"');
	}

	//{--} レイヤー検索
	var VisibleOnly	= data.VisibleOnly;
	var typename	= _config.getListValue("Target");
	var found_layer	= LayerUtil.each(
		 data.AsAllLayers ? activeDocument.layers : _config.global.targetParent.layers
		,function(layer){
			if(!_config.isProgressing){
				throw new ConfigDialog.CancelProgressError();
			}
			if((!VisibleOnly || layer.visible) && (!typename || typename == layer.typename)){
				if(search_reg.test(layer.name)){
					return layer;
				}
			}
		}
		,data.AsSubLayerSets
		,eventHandler
	);

	//--? 発見レイヤーを通知
	if(found_layer){
		var visible	= found_layer.visible;
		activeDocument.activeLayer	= found_layer;
		found_layer.visible	= visible;
		_config.alertAs(
			(found_layer.typename == "LayerSet" ? MESSAGE_RESULT_SET_PREFIX : MESSAGE_RESULT_PREFIX) +
			'"' + StringUtil.getFullName(found_layer, '" / "', "LayerSet") + '"'
		);
	}else{
		throw new Error('"' + _config.data.SearchName + '"' + ERROR_NOT_FOUND_SUFFIX);
	}

};


