﻿/**

///	レイヤー名に応じてソートするスクリプト

	制作：彩藤神楽	(http://www.re-sonance.com/)


	Copyright (C) 2007-2008 Saitoh Kagura, All Rights Reserved.

 */

const SCRIPT_TITLE	= "レイヤー並び替え";
const VERSION		= "0.1.00";


///-------------------------------------------------------------
///	外部スクリプトファイル読み込み
//--------------------------------------------------------------
#include "./lib/Resonance.jsxinc"
#include "./lib/ScriptUtil.jsxinc"
#include "./lib/StringUtil.jsxinc"
#include "./lib/LayerUtil.jsxinc"
#include "./lib/ConfigDialog.jsxinc"


///-------------------------------------------------------------
///	グローバル
//--------------------------------------------------------------

//--> 基礎準備
#target photoshop
app.bringToFront();
const NEW_LINE	= Resonance.NEW_LINE;
const NEW_LINE2	= NEW_LINE + NEW_LINE;
const INDENT	= Resonance.INDENT;
const INDENT2	= INDENT + INDENT;
_resonance		= new Resonance(SCRIPT_TITLE, VERSION);

///> 確認ダイアログ
const MESSAGE_CONFIRM_SUFFIX		= NEW_LINE2 + "よろしいですか？";
const MESSAGE_CONFIRM_SORT_PREFIX	= "レイヤーを、" + NEW_LINE2;
const MESSAGE_CONFIRM_SORT_SUFFIX	= "に並べ替えます。";
const MESSAGE_CONFIRM_SORT_UP		= MESSAGE_CONFIRM_SORT_PREFIX +
	"[ 1 ]" + NEW_LINE + "[ 2 ]" + NEW_LINE + "[ 3 ]" + NEW_LINE2 +
	"と昇順" + MESSAGE_CONFIRM_SORT_SUFFIX;
const MESSAGE_CONFIRM_SORT_DOWN		= MESSAGE_CONFIRM_SORT_PREFIX +
	"[ 3 ]" + NEW_LINE + "[ 2 ]" + NEW_LINE + "[ 1 ]" + NEW_LINE2 +
	"と降順" + MESSAGE_CONFIRM_SORT_SUFFIX;
const MESSAGE_CONFIRM_SORT_RANDOM	= "レイヤーをランダム" + MESSAGE_CONFIRM_SORT_SUFFIX;
const TITLE_RESULT		= "レイヤーソート完了";
const MESSAGE_RESULT	= "並び替えが完了しました。";
const MESSAGE_SKIP		= "並び替えの必要はありませんでした。";


//> グローバルに扱う設定系オブジェクト
var _config;

///> 設定データ
var _configData	= {};
_configData.infoData	= {
	 _sortPrefix	: "並び替え範囲【 "
	,_sortSuffix	: " 】"
	,Type			: ["昇順(&U)", "降順(&D)", "ランダム(&R)"]
	,_Target		: "並び替え対象:"
};
_configData.listData	= {
	 Type			: [LayerUtil.sort.ASCENDING_ORDER, LayerUtil.sort.DESCENDING_ORDER, LayerUtil.sort.RANDOM_ORDER]
};
_configData.defaultData	= {
	 ScriptTitle	: SCRIPT_TITLE
	,ScriptVersion	: VERSION
	,Type			: 0
	,Target			: 0
};
_configData.widthData	= {
	_sort			: 300
};
_configData.heightData	= {
	Type			: 80
};


//--> 本体実行
main();


///-------------------------------------------------------------
///	メインフロー関数
//--------------------------------------------------------------

///{} 本体
function main() {

	//--? 起動時チェック
	try{
		ScriptUtil.inspectDocumentExists();
		ScriptUtil.inspectPhotoshopVersion();
		ScriptUtil.inspectDocumentFormat();
	}catch(e){
		_resonance.showError(e);
		return;
	}

	try{

		mainFlow();

	}catch(e){
		_resonance.showError(e);
	}

};


///{} メインフロー
function mainFlow() {

	//--> 設定読み込み
	loadConfig();

	//--> 設定ダイアログ表示
	if(_config.show()){

		//(!) OKの場合
		sortLayers();

	}

};


///{} 設定読み込みその編集用ダイアログを配置
function loadConfig() {

	//--> XMLデータとダイアログ設定を関連付けるクラスを生成
	_config	= new ConfigDialog(_configData);
	ScriptUtil.tries(_resonance.windowTitle, _config.connectXMLON, _config, [
		 _resonance.configFilePath
		,SCRIPT_TITLE
		,VERSION
	]);

	_config.title	= _resonance.windowTitle;

  _config.prepare	= function(){

	this.global.targetParent	= LayerUtil.getTargetParentLayer(activeDocument.activeLayer);

	/// レイアウト

	const column_opt	= this.setOptions({orientation:"column"});
	const row_opt		= this.setOptions({orientation:"row"});

	//--> メインウィンドウ生成
	this.create();

	//--> メイングループ
	this.addGroup("_main", column_opt);
		this.addPanel("_main/_sort", column_opt, this.infoData._sortPrefix + StringUtil.getFullName(this.global.targetParent, "/", "LayerSet") + this.infoData._sortSuffix);
			this.addGroup("_main/_sort/Type", row_opt);
				this.addRadioButtonItem(undefined, {helpTip:MESSAGE_CONFIRM_SORT_UP});
				this.addRadioButtonItem(undefined, {helpTip:MESSAGE_CONFIRM_SORT_DOWN});
				this.addRadioButtonItem(undefined, {helpTip:MESSAGE_CONFIRM_SORT_RANDOM});

	this.changeParentGroup("_main/");
	this.addConfirmCheckBox();
	this.addSpace();

	//--> 基本ボタン
	this.addGroup("_main/_button", row_opt);
		this.addOKButton();
		this.addCancelButton();
		this.addApplyButton();


	/// イベントハンドラ

	var eventListener	= {};

	//e{} 対象レイヤー取得ボタンイベント
	eventListener.onClickOK	= function(conf, item){
		switch(conf.getListValue("Type")){
			case LayerUtil.sort.DESCENDING_ORDER:
				conf.confirmMessage	= MESSAGE_CONFIRM_SORT_DOWN;
				break;
			case LayerUtil.sort.RANDOM_ORDER:
				conf.confirmMessage	= MESSAGE_CONFIRM_SORT_RANDOM;
				break;
			case LayerUtil.sort.ASCENDING_ORDER:
			default:
				conf.confirmMessage	= MESSAGE_CONFIRM_SORT_UP;
				break;
		}
		conf.confirmMessage	+= MESSAGE_CONFIRM_SUFFIX;

		return true;

	};

	//--> イベントリスナーを登録
	this.setListener(eventListener);

  };

};


///{} レイヤーソート
function sortLayers() {

	if(
		LayerUtil.sortLayers(
			 _config.global.targetParent
			,_config.getListValue("Type")
		)
	){
		//(!) 並べ替え適用
		_config.alertAs(MESSAGE_RESULT, TITLE_RESULT);
	}else{
		//(!) 並べ替え不適用
		_config.alertAs(MESSAGE_SKIP, TITLE_RESULT);
	}

};


