﻿/**

///	ドキュメントファイル連結スクリプト

	制作：彩藤神楽	(http://www.re-sonance.com/)


	Copyright (C) 2007-2008 Saitoh Kagura, All Rights Reserved.

 */

const SCRIPT_TITLE	= "ファイル連結";
const VERSION		= "0.1.00";


///-------------------------------------------------------------
///	外部スクリプトファイル読み込み
//--------------------------------------------------------------
#include "./lib/Resonance.jsxinc"
#include "./lib/ScriptUtil.jsxinc"
#include "./lib/LayerUtil.jsxinc"
#include "./lib/ConfigDialog.jsxinc"


///-------------------------------------------------------------
///	グローバル
//--------------------------------------------------------------

//--> 基礎準備
#target photoshop
app.bringToFront();
const NEW_LINE	= Resonance.NEW_LINE;
const NEW_LINE2	= NEW_LINE + NEW_LINE;
const INDENT	= Resonance.INDENT;
_resonance		= new Resonance(SCRIPT_TITLE, VERSION);

///> 新規生成するPSDドキュメント名
const NEW_DOCUMENT_NAME	= "連結ドキュメント";

///> ダイアログ用定数
const CONFIRM_SUFFIX	= NEW_LINE2 + "よろしいですか？";
const CONFIRM_NEW_DOC	= "ドキュメント群を新規ドキュメントに連結します。";
const CONFIRM_ACTIVE_PREFIX	= "ドキュメント群を【 ";
const CONFIRM_ACTIVE_SUFFIX	= " 】に連結します。";
const CONFIRM_AS_CLOSES	= NEW_LINE2 + "元のドキュメント群は連結後に破棄されます。";
const TITLE_RESULT		= "ドキュメント連結完了";
const MESSAGE_RESULT	= "ドキュメントの連結が完了しました。";
const MESSAGE_RESULT_LAYER		= NEW_LINE2 + "連結レイヤー数" + INDENT;
const MESSAGE_RESULT_LAYER_SET	= NEW_LINE + "連結レイヤーセット数" + INDENT;

///> エラーメッセージ定数
const ERROR_TARGET_NOT_EXISTS	= "連結元レイヤーが存在しませんでした。";

//> グローバルに扱う設定系オブジェクト
var _config;

///> 設定データ
var _configData	= {};
var INDEX_EXPORT_NEW_DOC	= 0;
_configData.infoData	= {
	 _target		: "対象ドキュメント群:"
	,_recover		: "元に戻す(&R)"
	,_activeDoc		: "現在のアクティブドキュメント"
	,_exportDoc		: "連結先ドキュメント"
	,_config		: "詳細設定"
	,ExportDocument	: ["新規ドキュメント(&N)", "現在アクティブなドキュメント(&D)"]
	,EmptyExcepted	: "空レイヤーは除外(&E)"
	,VisibleOnly	: "表示レイヤーのみコピー(&V)"
	,AsCloses		: "連結後に元ドキュメントを閉じる(&C)"

};
_configData.defaultData	= {
	 ScriptTitle	: SCRIPT_TITLE
	,ScriptVersion	: VERSION
	,Target			: []
	,ExportDocument	: 0
	,EmptyExcepted	: true
	,VisibleOnly	: true
	,AsCloses		: false
	,Reversed		: false
};
_configData.widthData	= {
	 _main			: 360
	,Target			: 260
	,"_activeDoc,_exportDoc,_activeDoc"	: 380
};
_configData.heightData	= {
	 Target			: 160
};


//--> 本体実行
main();


///-------------------------------------------------------------
///	メインフロー関数
//--------------------------------------------------------------

///{} 本体
function main() {

	//--? 起動時チェック
	try{
		ScriptUtil.inspectDocumentExists(2);
		ScriptUtil.inspectPhotoshopVersion();
	}catch(e){
		_resonance.showError(e);
		return;
	}

	try{

		mainFlow();

	}catch(e){
		_resonance.showError(e);
	}

};


///{} メインフロー
function mainFlow() {

	//--> 設定読み込み
	loadConfig();

	//--> ダイアログを表示して実行
	if(_config.show()){

		//(!) OK
		mergeDocuments();

	}

};


///{} 設定読み込みその編集用ダイアログを配置
function loadConfig() {

	//--> XMLデータとダイアログ設定を関連付けるクラスを生成
	_config	= new ConfigDialog(_configData);
	ScriptUtil.tries(_resonance.windowTitle, _config.connectXMLON, _config, [
		 _resonance.configFilePath
		,SCRIPT_TITLE
		,VERSION
	]);

	_config.title	= _resonance.windowTitle;

  _config.prepare	= function(){

	//{--} 対象取得
	var docNames	= [];
	var len	= documents.length;
	for(var i = 0; i < len; i++){
		docNames[i]	= documents[i].name;
	}
	docNames.sort();
	docNames.reverse();

	/// レイアウト

	const column_opt	= this.setOptions({orientation:"column"});
	const row_opt		= this.setOptions({orientation:"row"});
	const config_opt	= this.setOptions({orientation:"column", alignChildren:"left"});

	//--> メインウィンドウ生成
	this.create();

	//--> メイングループ
	this.addGroup("_main", column_opt);

	//--> 対象ドキュメント表示リストボックス
	this.addGroup("_main/_export", {orientation:"column", alignment:"right"});
		this.addPanel("_main/_export/_exportDoc");
			this.addGroup("_main/_export/_exportDoc/ExportDocument");
				this.addRadioButtonItem();
				this.addRadioButtonItem();
		this.addPanel("_main/_export/_activeDoc");
			this.addStaticText('"' + activeDocument.name + '"');

	//--> 対象ドキュメント表示リストボックス
	this.addGroup("_main/_list", row_opt);
		this.addGroup("_main/_list/_menu", {orientation:"column", alignChildren:"right"});
			this.addText("_target");
			this.ascend();
		var target_lbx	= this.addEditListBox("Target", undefined, "onChangeList", docNames, true);
		this.global.target_lbx	= target_lbx;
		this.changeParentGroup("_main/_list/_menu/");
			this.addSpace();
			target_lbx.addReverseButton(undefined, undefined, "onClickReverse");
			target_lbx.addInvertButton();
			target_lbx.addExtractButton();

	//--> 設定
	this.addPanel("_main/_config", row_opt);
		this.addGroup("_main/_config/_custom", config_opt);
			this.addCheckBox("EmptyExcepted");
			this.addCheckBox("VisibleOnly");
			this.addCheckBox("AsCloses");
		this.addGroup("_main/_config/_custom2", config_opt);
			this.addQuickSelectCheckBox();
			this.addConfirmCheckBox();

	this.changeParentGroup("_main/");
		this.addSpace();

	//--> 基本ボタン
	this.addMainButton("_main/_button", row_opt, true, true);


	/// イベントハンドラ

	var eventListener	= {};

	///e{} 対象リストイベント
	eventListener.onChangeList	= function(conf, item){
		if(item.selection && item.selection.length <= 1 && item.extract_btn.text == ConfigDialog.button.EXTRACT_LIST){
			item.extract_btn.enabled	= false;
		}
	};

	///e{} 逆順ボタンイベント
	eventListener.onClickReverse	= function(conf, item){
		if(conf.showed){
			conf.data.Reversed	= !conf.data.Reversed;
		}
	};

	///e{} OKボタンイベント
	eventListener.onClickOK	= function(conf, item){

		conf.confirmMessage	= (conf.data.ExportDocument == INDEX_EXPORT_NEW_DOC ?
				CONFIRM_NEW_DOC
				:
				CONFIRM_ACTIVE_PREFIX + activeDocument.name + CONFIRM_ACTIVE_SUFFIX
			) +
			(conf.data.AsCloses ? CONFIRM_AS_CLOSES : "") +
			CONFIRM_SUFFIX;

	};

	//--> イベントリスナーを登録
	this.setListener(eventListener);

	if(this.data.Reversed){
		target_lbx.reverse_btn.onClick();
	}

  };

};


///{a} ドキュメント名リストから対象ドキュメントリストを取得
function getTargetDocuments(docNames) {

	//{--} 全ドキュメントを取得
	var docs	= [];
	var len		= documents.length;
	for(var i = 0; i < len; i++){
		docs[i]	= documents[i];
	}

	//{o} ドキュメント名からドキュメントを取得	※getByNameでは同名ドキュメントがある時区別がつかない
	var getTargetDocument	= function(docName){
		var len	= docs.length;
		for(var i = 0; i < len; i++){
			if(docs[i].name == docName){
				var doc	= docs[i];
				docs.splice(i, 1);
				return doc;
			}
		}
	}

	//{--} ドキュメント名リストに登録されているものを対象リストに順次追加
	var target_docs	= [];
	var len2	= docNames.length;
	for(var j = 0; j < len2; j++){
		target_docs.unshift(getTargetDocument(docNames[j]));
	}

	return target_docs;

};


///{} ファイル連結
function mergeDocuments() {

	var data			= _config.data;

	var target_lbx		= _config.global.target_lbx;
	var target_docs		= getTargetDocuments(target_lbx.toArray(data.quickSelected && target_lbx.selection && true));

	var isNewDoc		= (data.ExportDocument == INDEX_EXPORT_NEW_DOC);
	var VisibleOnly		= data.VisibleOnly;
	var EmptyExcepted	= data.EmptyExcepted;
	var AsCloses		= data.AsCloses;

	var temp_doc		= isNewDoc ? LayerUtil.duplicateEmptyDocument(NEW_DOCUMENT_NAME) : activeDocument;
	var default_layer	= isNewDoc ? temp_doc.layers[0] : null;
	var isPixelInLayer	= LayerUtil.isPixelInLayer;

	var countOfLayer	= 0;
	var countOfLayerSet	= 0;
	var docName;
	var singled;
	var layer;
	var clone_layer;
	var layers;
	var visible;
	var j;
	var k;
	var len2;
	var len	= target_docs.length;
	for(i = 0; i < len; i++){
		if(target_docs[i] == temp_doc){
			continue;
		}
		activeDocument	= target_docs[i];
		docName	= activeDocument.name.replace(/\.[^\\\/\.]*$/, "");
		layers	= activeDocument.layers;
		len2	= layers.length;
		if(!len2){
			continue;
		}
		singled	= (len2 == 1);
		k	= 1;
		for(j = len2 - 1; j >= 0; j--){
			layer	= layers[j];
			//--? 表示レイヤーのみコピーするか
			if(VisibleOnly && !layer.visible){
				continue;
			}
			//--? 中身が存在するレイヤーのみコピーするか
			if(EmptyExcepted && !isPixelInLayer(layer, true)){
				continue;
			}
			visible		= layer.visible;
			clone_layer	= layer.duplicate(temp_doc);
			activeDocument	= temp_doc;
			//--? 状況次第ではレイヤー名にドキュメント名を適用する
			if(singled || clone_layer.name.search(/^レイヤー \d+/) != -1){
				clone_layer.name	= docName + (k > 1 ? " " + k : "");
				k++;
			}
			clone_layer.visible	= visible;
			activeDocument	= target_docs[i];
			if(clone_layer.typename == "LayerSet"){
				countOfLayerSet++;
			}else{
				countOfLayer++;
			}
		}
		if(AsCloses){
			target_docs[i].close(SaveOptions.DONOTSAVECHANGES);
		}
	}

	activeDocument	= temp_doc;
	if(temp_doc.layers.length == 1){
		if(isNewDoc){
			temp_doc.close(SaveOptions.DONOTSAVECHANGES);
		}
		throw new Error(ERROR_TARGET_NOT_EXISTS);
	}

	if(default_layer)	default_layer.remove();
	if(clone_layer)		clone_layer.visible	= visible;	//※最後にactiveLayerになるのが非表示レイヤーだとここで改めて復元する必要がある

	_config.alertAs(MESSAGE_RESULT + MESSAGE_RESULT_LAYER + countOfLayer + MESSAGE_RESULT_LAYER_SET + countOfLayerSet, TITLE_RESULT);

};


